# This script downloads all the available raw data used. It must be
# run in an environment with an internet connection i.e. not on a worker node.

suppressMessages({
  library(TENxPBMCData)
  library(scRNAseq)
  library(AnnotationHub)
  library(zellkonverter)
  library(tibble)
  library(EnsDb.Mmusculus.v79)
  library(SingleCellExperiment)
  library(readr)
  library(dplyr)
})

raw_lawlor <- LawlorPancreasData()
saveRDS(raw_lawlor, here::here("data", "raw_data", "lawlor.rds"))

raw_pbmc3k <- TENxPBMCData(dataset = "pbmc3k")
saveRDS(raw_pbmc3k, here::here("data", "raw_data", "pbmc3k.rds"))

raw_zeisel <- ZeiselBrainData()
saveRDS(raw_zeisel, here::here("data", "raw_data", "zeisel.rds"))

library(reticulate)
sc <- import("scanpy")
paul_adata <- sc$datasets$paul15()
paul_sce <- AnnData2SCE(paul_adata)
saveRDS(paul_sce, here::here("data", "raw_data", "paul.rds"))

raw_zhao <- ZhaoImmuneLiverData()
saveRDS(raw_zhao, here::here("data", "raw_data", "zhao.rds"))

edb <- AnnotationHub()[["AH73881"]]
saveDb(edb, here::here("data", "edb.sqlite"))

# astrocyte data

c2l_all_cells_sce <- readH5AD(here::here(
  "data",
  "downloaded_data",
  "all_cells_20200625.h5ad"
))

c2l_astrocytes_sce <- readH5AD(here::here(
  "data",
  "downloaded_data",
  "mouse_brain_snRNA_with_UMAP_astrocytes_cellxgene.h5ad"
))

c2l_cell_labels <- tibble(read.csv(here::here(
  "data",
  "downloaded_data",
  "snRNA_annotation_astro_subtypes_refined59_20200823.csv"
)))

colnames(c2l_cell_labels) <- c("cell_label", "annotation")
c2l_cell_labels$is_astro <- substr(c2l_cell_labels$annotation, 1, 5) == "Astro"

astro_cell_labels <- c2l_cell_labels$cell_label[c2l_cell_labels$is_astro]
astro_annotations <- c2l_cell_labels$annotation[c2l_cell_labels$is_astro]

astro_sce <- c2l_all_cells_sce[, astro_cell_labels]
colLabels(astro_sce) <- astro_annotations

counts(astro_sce) <- assay(astro_sce, "X")
assay(astro_sce, "X") <- NULL

rownames(astro_sce) <- rowData(astro_sce)$SYMBOL
astro_sce <- astro_sce[rownames(c2l_astrocytes_sce), ]

# Permute the cells into the same ordering.
c2l_astrocytes_sce <- c2l_astrocytes_sce[, colnames(astro_sce)]
logcounts(astro_sce) <- assay(c2l_astrocytes_sce, "X")

saveRDS(astro_sce, here::here("data", "raw_data", "astrocyte.rds"))

# SMART-seq3 PBMC data

umi_counts <- read_table(
  here::here("data", "downloaded_data", "HCA.UMIcounts.PBMC.txt" )
)

pbmc_anno <- read_tsv(
  here::here("data", "downloaded_data", "Smart3.PBMC.annotated.txt")
)

# all(colnames(umi_counts) == pbmc_anno$cellID)

counts_mat <- as.matrix(umi_counts[, -1])
rownames(counts_mat) <- umi_counts[, 1][[1]]

ss3_pbmc <- SingleCellExperiment(assays = list(counts = counts_mat))

colData(ss3_pbmc) <- DataFrame(cellID = colnames(ss3_pbmc))

# Need to left_join because the number row/cells does not exactly line up.
colData(ss3_pbmc) <- colData(ss3_pbmc) |>
  as_tibble() |>
  left_join(pbmc_anno, by = "cellID") |>
  DataFrame()

saveRDS(ss3_pbmc, here::here("data", "raw_data", "ss3_pbmc.rds"))
