# Run the specified method.

suppressMessages({
  library(R.utils)
  library(jsonlite)
  library(SingleCellExperiment)
})

# Get the passed arguments from the command line via Snakemake.
args <- commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging
print(args)

# Extract the elements of the passed argument.
data <- readRDS(args$sim)
meth_pars <- as.list(fromJSON(args$meth_pars))
# TODO: Improve argument names. These come from the snakefile.
method_file <- args$fun
result_file <- args$res

# This file contains functions used to validate the output of the the
# marker gene selection methods.
source(here::here("code", "run-utils.R"))

# In this project the file run_{method} will contain a function `run_{method}`
# which takes two arguments: a SingleCellExperiment object and a named list of
# parameters/arguments for the method.
source(method_file)
func <- gsub("(.R)", "", basename(method_file))
func <- gsub("-", "_", func)
result <- get(func)(data, meth_pars)

# TODO: Add more context the the result.

# Save the results.
saveRDS(result, result_file)
